#ifndef __ALIGN_H__
#define __ALIGN_H__

#include "image_types.h"
#include "kdtree.h"
#include <vector>

namespace s3d_merge
{

enum AlignmentParameters
{
    ALIGN_ANGLE=0,
    ALIGN_TRANSLATE_X,
    ALIGN_TRANSLATE_Y,
    ALIGN_NUM_PARAMETERS
};

typedef spectral::Point<double, 2> Point2D;
typedef spectral::KDTree<double, 2> KDTree2D;

typedef spectral::Point<double, 6>  PatternNode;
typedef spectral::KDTree<double, 6> PatternTree;


class StarRef
{
public:
    StarRef()
        : m_value(0)
        , m_pixels(0)
    {
        m_point[0] = m_point[1] = 0.0;
    }

    StarRef(double x, double y, double value, size_t pixels)
        : m_value(value)
        , m_pixels(pixels)
    {
        m_point[0] = x;
        m_point[1] = y;
    }

    const Point2D &GetPoint(void) const { return m_point; }
private:
    Point2D m_point;
    double m_value;
    size_t m_pixels;
};

void FindStars(const Image16 *img, std::vector<StarRef> &stars);

KDTree2D * CreateKDTree(const std::vector<StarRef> &stars);

PatternNode GeneratePatternNode(const std::vector<Point2D> &points, const KDTree2D *tree, size_t index);

void
GeneratePatternNodes(const std::vector<Point2D> &points,
                     const KDTree2D *tree,
                     std::vector<PatternNode> &out);

bool
FitStars(const PatternTree *ref_pattern_tree,
         const std::vector<PatternNode> &ref_patterns,
         const KDTree2D *ref_tree,
         const std::vector<Point2D> &ref_points,
         const std::vector<Point2D> &cur_points,
         double *alignment);

}

#endif
