#ifndef __INTEGRAL_HISTOGRAM_H__
#define __INTEGRAL_HISTOGRAM_H__
#include "image.h"
#include "image_types.h"
#include "stdint.h"

namespace s3d_merge {

//! An integral histogram implementation.
class IntegralHistogram : public spectral::image<uint32_t>
{
public:
    IntegralHistogram(uint32_t bins,
                      const Image8 &img,
                      uint32_t x0,
                      uint32_t y0,
                      uint32_t width,
                      uint32_t height,
                      uint32_t(*pixel_to_bin)(uint32_t, 
                                              uint32_t,
                                              const Image8 *,
                                              void *),
                      void *data = NULL);

    IntegralHistogram(uint32_t bins,
                      const Image8 &img, 
                      uint32_t(*pixel_to_bin)(uint32_t, 
                                              uint32_t,
                                              const Image8 *,
                                              void *),
                      void *data = NULL);

    virtual ~IntegralHistogram();

    //! Populate a histogram for a region.
    void GetHistogram(uint32_t x1,
                      uint32_t y1,
                      uint32_t x2,
                      uint32_t y2,
                      uint32_t *result) const;
protected:

    void
    BuildHistogram(uint32_t bins,
                   const Image8 &img,
                   uint32_t x0,
                   uint32_t y0,
                   uint32_t width,
                   uint32_t height,
                   uint32_t(*pixel_to_bin)(uint32_t, 
                                           uint32_t,
                                           const Image8 *,
                                           void *),
                   void *data = NULL);
};


}

#endif
